# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String


class LotusVircomBase(Splitted):
    SEPARATOR = "|"


class LotusVircomOrder(LotusVircomBase):
    FIELDS = (
        String("sample_code"),
        Multiple("tests", (String("test_code"),), separator="|"),
    )

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class LotusVircomQuery(LotusVircomBase):
    FIELDS = (Multiple("queries", (String("tube"),), separator="|"),)

    def parse(self, line):
        fields = self.FIELDS
        setattr(self, fields[0].name, line.split("|"))


class LotusVircomResult(LotusVircomBase):
    FIELDS = [
        String("test_code"),
        String("sample_code"),
        String("result_measurement"),
        String("numerical_result"),
        String("units"),
        String("decimal_separator"),
        String("strip_position"),
        String("well"),
        String("units_transformation"),
        String("result_interpretation"),
        String("error"),
        String("individually_accepted_value"),
    ]


class LotusVircomExtentResult(LotusVircomBase):
    FIELDS = [
        String("test_code"),
        String("sample_code"),
        String("result_measurement"),
        String("numerical_result"),
        String("units"),
        String("decimal_separator"),
        String("well"),
        String("strip_position"),
        String("unit_transformation"),
        String("result_interpretation"),
        String("error"),
        String("individually_accepted_value"),
        String("qc1_status"),
        String("qc2_status"),
        String("cal1_name"),
        String("cal2_name"),
        String("cal1_measurement"),
        String("cal2_measurement"),
        String("qc1_validation_rule"),
        String("qc2_validation_rule"),
        String("reagent_lot"),
        String("exp_date"),
        String("sample_reading"),
        String("messages_analysis"),
    ]


class LotusVircomMultiple(list):
    EXTENDED_RESULT = 99
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: LotusVircomOrder,
        MessageV2.RESULT: LotusVircomResult,
        MessageV2.QUERY: LotusVircomQuery,
        EXTENDED_RESULT: LotusVircomExtentResult,
    }
    SEPARATOR = "|"

    def __init__(self, **kwargs):
        self._locked = False
        self.entete_order = kwargs.pop("entete_order", "")
        super(LotusVircomMultiple, self).__init__(**kwargs)

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.split("\n"):
            if not len(line):
                continue
            if line[-1] == self.SEPARATOR:
                line = line[:-1]
            champs = line.split(self.SEPARATOR)
            if len(champs) == 12:
                type_message = MessageV2.RESULT
            elif len(champs) == 24:
                type_message = self.EXTENDED_RESULT
            elif line == self.entete_order:
                type_message = MessageV2.ORDER
            else:
                type_message = MessageV2.QUERY
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        msg = ""
        if self[0] and isinstance(self[0], LotusVircomOrder):
            msg = self.entete_order + "\r\n"
        msg += "\r\n".join([e.to_string() for e in self])
        return msg


class Driver(MessageV2):
    SPEC = {"code": "lotus_vircom", "label": "Lotus Vircom ORGENTEC"}

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: LotusVircomOrder,
        MessageV2.RESULT: LotusVircomResult,
    }
    ENTETE_ORDER = "LEGIO.PN.1 IgM|HSV-1 IgG|M.PNEUMONIAE IgM|PARAINFLUENZA3 IgG|R. CONORII IgM|HYDATIDOSIS IgG|"
    OPTIONS = [
        MessageV2.Option(
            "lotus_vircom_entete_order",
            label="Analyses de l'entete du message order",
            default=ENTETE_ORDER,
        )
    ]

    def __init__(self, **kwargs):
        self.entete_order = kwargs.pop("lotus_vircom_entete_order", self.ENTETE_ORDER)
        super(Driver, self).__init__(**kwargs)

    def create(self):
        return LotusVircomMultiple(entete_order=self.entete_order)

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = LotusVircomMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        message = self.create()
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) == 0:
            return None
        for sample, action in samples:
            order_message = message.add(MessageV2.ORDER, sample_code=sample.sample_id)
            for order in sample["orders"]:
                order_message.tests.add(test_code=order["code"])
        return message

    def decode_message(self, message):
        if message and isinstance(
            message[0], (LotusVircomResult, LotusVircomExtentResult)
        ):
            # Format RESULT FILE
            worklist = Worklist(context="result")
            for m in message:
                sample = worklist.get_or_create(sample_id=m.sample_code, type="patient")
                flags = []
                if m.individually_accepted_value:
                    flags.append(m.individually_accepted_value)
                if m.error:
                    flags.append(m.error)
                if m.result_measurement:
                    resu = sample.results.add(
                        code="{} MEASUREMENT".format(m.test_code),
                        value=m.result_measurement.replace(",", "."),
                        flags=flags,
                    )
                    if isinstance(m, LotusVircomExtentResult) and m.messages_analysis:
                        resu["comments"] = [m.messages_analysis]
                if m.numerical_result:
                    resu = sample.results.add(
                        code="{} NUMERICAL".format(m.test_code),
                        value=m.numerical_result.replace(",", "."),
                        flags=flags,
                    )
                    if isinstance(m, LotusVircomExtentResult) and m.messages_analysis:
                        resu["comments"] = [m.messages_analysis]
                if m.result_interpretation:
                    resu = sample.results.add(
                        code="{} INTERPRETATION".format(m.test_code),
                        value=m.result_interpretation,
                        flags=flags,
                    )
                    if isinstance(m, LotusVircomExtentResult) and m.messages_analysis:
                        resu["comments"] = [m.messages_analysis]
        elif message and isinstance(message[0], LotusVircomQuery):
            worklist = Worklist(context="query")
            for m in message:
                for tube in m.queries:
                    worklist.get_or_create(type="patient", sample_id=tube.strip())
        return worklist
